${eCAP_AV_SDK_BOILERPLATE}

#ifndef ${eCAP_AV_SDK_GUARD}_XACTION_H
#define ${eCAP_AV_SDK_GUARD}_XACTION_H

#include <libecap/adapter/xaction.h>
#include <libecap/host/host.h>
#include <libecap/host/xaction.h>
#include <libecap/common/memory.h>
#include <libecap/common/message.h>
#include <libecap/common/header.h>
#include <libecap/common/names.h>

#include "Antivirus.h"

namespace Adapter {

class Service;
using libecap::size_type;

class Answers;
class Xaction;

typedef libecap::shared_ptr<Adapter::Xaction> XactionPointer;

// Antivirus Answer that can be async-delivered to Xaction via Queue
class MyAnswer: public Answer {
	public:
		MyAnswer(const std::string & vbFileName, const XactionPointer &anX, Answers *answers);
		virtual ~MyAnswer();

		/* Answer API */
		virtual void deliver();

		XactionPointer xaction;

	private:
		Answers *answers; // delivery queue
};

// eCAP adapter transaction that scans the virgin message using Antivirus API
class Xaction: public libecap::adapter::Xaction, public Antivirus::User 
{
	public:
		Xaction(libecap::shared_ptr<Service> aService, libecap::host::Xaction *aHostX);
		virtual ~Xaction();

		// meta-information for the host transaction
		virtual const libecap::Area option(const libecap::Name &name) const;
		virtual void visitEachOption(libecap::NamedValueVisitor &visitor) const;

		// lifecycle
		virtual void start();
		virtual void stop();
		virtual void resume(); // check for decision

		// adapted body transmission control
		virtual void abDiscard();
		virtual void abMake();
		virtual void abMakeMore();
		virtual void abStopMaking();

		// adapted body content extraction and consumption
		virtual libecap::Area abContent(size_type offset, size_type size);
		virtual void abContentShift(size_type size);

		// virgin body state notification
		virtual void noteVbContentDone(bool atEnd);
		virtual void noteVbContentAvailable();

		// Antivirus::User API
		virtual void onAnswer(const Answer &answer);

		// give host control after async analysis
		void tellHostToResume(Answer *aAnswer);

		XactionPointer self;

	protected:
		bool shouldExamine(); // decide whether to receive and scan the message

		void handleHuge(const char *where); // deal with over-the-limit vb size
		void handleError(const char *where);

		void adaptContent(std::string &chunk); // converts vb to ab
		void stopVb(); // stops receiving vb (if we are receiving it)
		libecap::host::Xaction *lastHostCall(); // clears hostx
		void useVirgin();  // tell host to use virgin message
		void useStored(); // tell host to use stored message
		void allowAccess(); // tell host to forward the message
		void blockAccess(); // tell host to deny user access
		void getUri();
				
		void onClean();
		void onVirus(const char *virusName);
		void onError(const char *error);

		void debugAction(const std::string &action, const char *detail = 0);

		bool open(); // creates staging file
		void close(); // removes staging file
		FILE *abFileX(); // adapted body file pointer; throws if nil

	private:
		libecap::shared_ptr<const Service> service; // configuration access
		libecap::host::Xaction *hostx; // Host transaction rep
		libecap::Area uri; // Request-URI from headers, for logging

		Answer *answer; // asynchronously delivered answer we need to process
		std::string virusId; // Antivirus-reported "virus name" or empty

		FILE *vbFile; // stored virgin body
		std::string vbFileName; // temporary filename for vb storage
		Size vbOffset; // virgin body bytes seen and consumed by adapter
		Size abOffset; // adapted body bytes "consumed" by host

		typedef enum { opUndecided, opOn, opComplete, opNever } OperationState;
		OperationState receivingVb; // receiving of virgin body state
		OperationState sendingAb; // sending of adapted body state

		bool vbComplete; // got entire virgin body
};

}

#endif
